import { JSONRPCClient, JSONRPCServer, JSONRPCServerAndClient } from "json-rpc-2.0";
import { ConnectionState } from "./types.js";
export class RpcRequest {
    origin;
    version;
    payload;
    constructor(origin, payload = {}, version = 1) {
        this.origin = origin;
        this.version = version;
        this.payload = payload;
    }
}
export var AWCStreamChannelState;
(function (AWCStreamChannelState) {
    AWCStreamChannelState[AWCStreamChannelState["CONNECTING"] = 0] = "CONNECTING";
    AWCStreamChannelState[AWCStreamChannelState["OPEN"] = 1] = "OPEN";
    AWCStreamChannelState[AWCStreamChannelState["CLOSING"] = 2] = "CLOSING";
    AWCStreamChannelState[AWCStreamChannelState["CLOSED"] = 3] = "CLOSED";
})(AWCStreamChannelState || (AWCStreamChannelState = {}));
export class ArchethicWalletClient {
    origin;
    client;
    _channel;
    _connectionStateEventTarget;
    _rpcNotificationEventTarget;
    static _instance;
    constructor(channel) {
        this.origin = { name: "" };
        this._connectionStateEventTarget = new EventTarget();
        this._rpcNotificationEventTarget = new EventTarget();
        this._channel = channel;
    }
    static init(channel) {
        this._instance = new this(channel);
    }
    static get instance() {
        if (!this._instance) {
            throw 'ArchethicWalletClient must be init before using it';
        }
        return this._instance;
    }
    setOrigin(origin) {
        this.origin = origin;
        return this;
    }
    _dispatchConnectionState(state) {
        const connectionState = state ?? this.connectionState;
        console.log(`Connection state updated : ${connectionState}`);
        this._connectionStateEventTarget.dispatchEvent(new Event(connectionState));
    }
    async connect() {
        return new Promise(async (resolve, reject) => {
            console.log('Connection attempt');
            if (this.connectionState != ConnectionState.Closed) {
                return reject(new Error("Connection already established. Cancelling new connection request"));
            }
            this.client = new JSONRPCServerAndClient(new JSONRPCServer(), new JSONRPCClient((request) => {
                try {
                    this._channel?.send(JSON.stringify(request));
                    return Promise.resolve();
                }
                catch (error) {
                    return Promise.reject(error);
                }
            }));
            this.client.addMethod("addSubscriptionNotification", (request) => {
                const notification = {
                    subscriptionId: request["subscriptionId"],
                    data: request["data"]
                };
                this._rpcNotificationEventTarget.dispatchEvent(new CustomEvent(notification.subscriptionId, {
                    detail: notification.data
                }));
            });
            this._channel.onReceive = async (data) => {
                this.client?.receiveAndSend(JSON.parse(data));
            };
            this._channel.onClose = async (reason) => {
                this.client?.rejectAllPendingRequests(`Connection is closed (${reason}).`);
                this.client = undefined;
                this._dispatchConnectionState();
            };
            this._channel.onReady = async () => {
                resolve();
                this._dispatchConnectionState();
            };
            this._channel.connect();
            this._dispatchConnectionState();
        });
    }
    async close() {
        this._dispatchConnectionState(ConnectionState.Closing);
        await this._channel?.close();
    }
    _ensuresConnectionAlive() {
        if (this.client == null || this.connectionState != ConnectionState.Open)
            throw new Error("RPC connection must be alive.");
    }
    onAccountUpdate(accountName, listener) {
        return this._subscribe("subscribeAccount", {
            name: accountName
        }, listener);
    }
    _subscribe(method, data, listener) {
        this._ensuresConnectionAlive();
        return this.client?.request(method, new RpcRequest(this.origin, data)).then((result) => {
            const subscription = {
                id: result["subscriptionId"],
                eventListener: (event) => listener(event.detail)
            };
            this._rpcNotificationEventTarget.addEventListener(subscription.id, subscription.eventListener);
            return subscription;
        });
    }
    unsubscribe(subscription) {
        this._rpcNotificationEventTarget.removeEventListener(subscription.id, subscription.eventListener);
    }
    get connectionState() {
        switch (this._channel?.state) {
            case AWCStreamChannelState.CLOSING:
                return ConnectionState.Closing;
            case AWCStreamChannelState.CONNECTING:
                return ConnectionState.Connecting;
            case AWCStreamChannelState.OPEN:
                return ConnectionState.Open;
            default:
                return ConnectionState.Closed;
        }
    }
    onconnectionstatechange(listener) {
        this._connectionStateEventTarget.addEventListener(ConnectionState.Connecting, () => {
            listener(ConnectionState.Connecting);
        });
        this._connectionStateEventTarget.addEventListener(ConnectionState.Open, () => {
            listener(ConnectionState.Open);
        });
        this._connectionStateEventTarget.addEventListener(ConnectionState.Closing, () => {
            listener(ConnectionState.Closing);
        });
        this._connectionStateEventTarget.addEventListener(ConnectionState.Closed, () => {
            listener(ConnectionState.Closed);
        });
        return this;
    }
    unsubscribeconnectionstatechange() {
        this._connectionStateEventTarget.removeEventListener(ConnectionState.Connecting, null);
        this._connectionStateEventTarget.removeEventListener(ConnectionState.Open, null);
        this._connectionStateEventTarget.removeEventListener(ConnectionState.Closing, null);
        this._connectionStateEventTarget.removeEventListener(ConnectionState.Closed, null);
        return this;
    }
    async getEndpoint() {
        this._ensuresConnectionAlive();
        return this.client?.request("getEndpoint", new RpcRequest(this.origin));
    }
    async sendTransaction(transaction) {
        this._ensuresConnectionAlive();
        return this.client?.request("sendTransaction", new RpcRequest(this.origin, transaction.toWalletRPC()));
    }
    async signTransactions(serviceName, pathSuffix, transactions) {
        this._ensuresConnectionAlive();
        const txs = transactions.map((tx) => tx.toWalletRPC());
        return this.client.request("signTransactions", new RpcRequest(this.origin, {
            serviceName: serviceName,
            pathSuffix: pathSuffix,
            transactions: txs
        })).then((result) => {
            for (let i = 0; i < result.signedTxs.length; i++) {
                transactions[i].setAddress(result.signedTxs[i].address);
                transactions[i].setPreviousSignatureAndPreviousPublicKey(result.signedTxs[i].previousSignature, result.signedTxs[i].previousPublicKey);
                transactions[i].setOriginSign(result.signedTxs[i].originSignature);
            }
            return transactions;
        });
    }
    async addService(name) {
        this._ensuresConnectionAlive();
        return this.client?.request("addService", new RpcRequest(this.origin, { name: name }));
    }
    async getAccounts() {
        this._ensuresConnectionAlive();
        return this.client?.request("getAccounts", new RpcRequest(this.origin)).then((result) => {
            return result["accounts"];
        });
    }
    async getCurrentAccount() {
        this._ensuresConnectionAlive();
        return this.client?.request("getCurrentAccount", new RpcRequest(this.origin));
    }
    async getServices() {
        this._ensuresConnectionAlive();
        return this.client?.request("getServicesFromKeychain", new RpcRequest(this.origin)).then((result) => {
            return result["services"];
        });
    }
    async keychainDeriveKeypair(serviceName, index = 0, pathSuffix = "") {
        this._ensuresConnectionAlive();
        return this.client?.request("keychainDeriveKeypair", new RpcRequest(this.origin, {
            serviceName: serviceName,
            index: index,
            pathSuffix: pathSuffix
        }));
    }
    async keychainDeriveAddress(serviceName, index = 0, pathSuffix = "") {
        this._ensuresConnectionAlive();
        return this.client?.request("keychainDeriveAddress", new RpcRequest(this.origin, {
            serviceName: serviceName,
            index: index,
            pathSuffix: pathSuffix
        }));
    }
    onCurrentAccountChange(listener) {
        return this._subscribe("subscribeCurrentAccount", {}, listener);
    }
    refreshCurrentAccount() {
        this._ensuresConnectionAlive();
        this.client?.request("refreshCurrentAccount", new RpcRequest(this.origin));
    }
}
//# sourceMappingURL=wallet_rpc.js.map